///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Dateiname: MATHSTR.CPP                                                    //
//                                                                           //
// Autor:     Andreas Jger, Friedrich-Schiller-Universitt Jena             //
//                                                                           //
// System:    WIN_RWPM.EXE und WINLRWPM.EXE                                  //
//                                                                           //
// Beschreibung: String-Klasse, die komfortabel mit Zahlen, Vektoren und     //
//            Matrizen umgehen kann                                          //
//                                                                           //
// Hinweise:                                                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <math.h>
#include <locale.h>

#include "mathutil.h"
#include "Vektor.h"
#include "MathStr.h"
#include "FktParser.h"

// Konstruktor, der aus einer long double-Zahl einen String macht
// Funktioniert zur Zeit nur fr den double-Wertebereich.
CMathString::CMathString(long double t, int ndig, bool /*scientific*/, int /*acceptpotenz*/) : CStdString()
{
	char buffer[40];
	sprintf(buffer, _T("%.*lG"), ndig, t);
	// Kommas in Punkte verwandeln (sollte eleganter gehen)
	for (unsigned int i=0; i<strlen(buffer); i++)
	{
		if (buffer[i]==',') buffer[i]='.';
	}
	*this = CStdString(buffer);
}

CMathString::CMathString(const CVektor& x, int ndig, bool scientific, char* delimiter,
	char* prefix, char* suffix) : CStdString(prefix)
{
	for (int i=1;i<=x.Dim;i++)
	{
		CStdString str = CMathString(x[i], ndig, scientific);
		*this += CMathString(x[i], ndig, scientific);
		if (i<x.Dim) 
		{
			*this += delimiter;
		}
	}
	*this += suffix;
}

CMathString::CMathString(const CMatrix& A, int ndig, bool scientific, char* delimiter,
	char* prefix, char* suffix, char* newline) : CStdString()
{
	for (int i=1;i<=A.Zeilen();i++)
	{
		*this += prefix;
		for (int j=1;j<=A.Spalten();j++)
		{
			*this += CMathString(A(i,j), ndig, scientific);
			*this += CStdString(j<A.Spalten() ? delimiter : suffix);
		}
		if (i<A.Zeilen()) 
		{
			*this += newline;
		}
	}
}

bool CMathString::Replace(int pos, int len, /*CStdString&*/const char* str)
{
	*this = Left(pos) + str + Mid(pos + len);
	return true;
};

int CMathString::Replace_All(const char* findstring, const char* Replacestring, int /*_tf*/)
{
	int anz = 0;
	//int tf = get_case_sensitive_flag();
	//set_case_sensitive(_tf);
	int op1 = Find(findstring);
	int op = op1;
	while (op1 != -1)
	{
		Replace(op, strlen(findstring), Replacestring);
		anz++;
		op += strlen(Replacestring);
		CStdString rest = Mid(op);
		if (rest.IsEmpty() || anz==10) return anz;
		op1 = rest.Find(findstring);
		op += op1;
	}
	//set_case_sensitive(tf);
	return anz;
}

long double CMathString::Value()
{
	CFunction<long double>* fkt = CFunction<long double>::Parse(*this);
	if (fkt)
	{
		long double y = fkt->Execute(0.0);
		delete fkt;
		return y;
	}
	else
		return 0.0;
}

long double CMathString::Value(const char* str)
{
	CMathString mstr(str);
	return mstr.Value();
}

CStdString CMathString::rational(long double x, long maxtest, long double TOL)
{
	char a[35];
#pragma warning ( disable : 4244 )
	long y = x;
#pragma warning ( default : 4244 )
	CStdString str;
	if (fabsl(y - x) < TOL)
	{
		str = CStdString(ltoa(y, a, 10));
		return str;
	}
	for (long p=2; p<=maxtest; p++)
	{
#pragma warning ( disable : 4244 )
		y = x*p;
#pragma warning ( default : 4244 )
		if (fabsl(y - p*x) < TOL)
		{
			str = CStdString(ltoa(y, a, 10));
			str += _T("/");
			str += CStdString(ltoa(p, a, 10));
			return str;
		}
	}
	return CMathString(x, 20);
}

CStdString CMathString::rationalTeX(long double x, long maxtest, long double TOL)
{
	char a[35];
#pragma warning ( disable : 4244 )
	long y = x;
#pragma warning ( default : 4244 )
	CStdString str;
	if (fabsl(y - x) < TOL)
	{
		str = CStdString(ltoa(y, a, 10));
		return str;
	}
	for (long p=2; p<=maxtest; p++)
	{
#pragma warning ( disable : 4244 )
		y = x*p;
#pragma warning ( default : 4244 )
		if (fabsl(y - p*x) < TOL)
		{
			str = _T("\\frac{");
			str += CStdString(ltoa(y, a, 10));
			str += _T("}{");
			str += CStdString(ltoa(p, a, 10));
			str += _T("}");
			return str;
		}
	}
	return CMathString(x, 20);
}

int  CMathString::Replace_All(CStdString& str, /*CStdString&*/const char* findstring, /*CStdString&*/const char* replacestring, int /*_tf*/)
{
	int anz = 0;
	//int tf = get_case_sensitive_flag();
	//set_case_sensitive(_tf);
	int op1 = str.Find(findstring);
	int op = op1;
	while (op1 != -1)
	{
		Replace(str, op, strlen(findstring), replacestring);
		anz++;
		op += strlen(replacestring);
		CStdString rest = str.Mid(op);
		if (rest.IsEmpty() || anz==10) return anz;
		op1 = rest.Find(findstring);
		op += op1;
	}
	//set_case_sensitive(tf);
	return anz;
}

bool CMathString::Replace(CStdString& str, int pos, int len, /*CStdString&*/const char* replstr)
{
	str = str.Left(pos) + replstr + str.Mid(pos + len);
	return true;
}

void CMathString::MakeUpper1()
{
	MakeUpper1(*this);
}

void CMathString::MakeLower1()
{
	MakeLower1(*this);
}

void CMathString::MakeUpper1(CStdString& str)
{
	str.MakeUpper();
	Replace_All(str, _T(""), _T(""));
	Replace_All(str, _T(""), _T(""));
	Replace_All(str, _T(""), _T(""));
	Replace_All(str, _T(""), _T("SS"));
}

void CMathString::MakeLower1(CStdString& str)
{
	str.MakeUpper();
	Replace_All(str, _T(""), _T(""));
	Replace_All(str, _T(""), _T(""));
	Replace_All(str, _T(""), _T(""));
}

CStdString CVektor::AsString()
{
	return CMathString(*this, 10, true);
}

CStdString CLinePoint::AsString()
{
	return CMathString(CVektor(*this), 10, true);
}

CStdString CMatrix::AsString()
{
	return CMathString(*this, 10, true);
}

